#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "macros.h"

#define NUM_SAMPLES 50

#define DEBOUNCE 2
// send = PA5

// receive_0 = PA0 (play/pause)
// receive_1 = PA1 (skip)
// receive_2 = PA2 (back)
// receive_3 = PA3 (plus)
// receive_4 = PA4 (minus)

int delay_times[] = {204, 186, 172, 155, 155};
//int thresholds[] = {1300, 1700?, 1000?, 750?, 3500?};
int thresholds[] = {1300, 1400, 900, 700, 3400};
char characters[] = {'s', 'n', 'b', 'p', 'm'};
char buttons_down[] = {0, 0, 0, 0, 0};

// LED = PA6

// clock in = pb2
// data out = pa7

#define DATA_OUT_LOW()      (PORTA &= ~(1 << PA7))
#define DATA_OUT_HIGH()     (PORTA |= (1 << PA7))
#define CLOCK_IN            (PINB & (1 << PB2))

void put_char(unsigned char byte_to_send)
{
    set(PORTA, PA6);
    while(!CLOCK_IN)
        ; // Wait for the clock to be released
    DATA_OUT_LOW(); // Tell the master that we're ready to send data.
    unsigned char mask;
    for (mask = 1 << 7; mask != 0; mask >>= 1) {
        while(CLOCK_IN)
            ; // Wait for the clock to go low
        clear(PORTA, PA6);
        if (byte_to_send & mask)
            DATA_OUT_HIGH();
        else
            DATA_OUT_LOW();
        while(!CLOCK_IN)
            ; // Wait for the clock to go high (which happens after the master samples data)
    }
    DATA_OUT_HIGH();
}

void put_int(int x) {

    if (x == 0) {
        put_char('0');
        return;
    }
    
    if (x < 0) {
        put_char('-');
        x = -x;
    }
    
    // Loop until we exceed the value to be printed
    int index = 1;
    for (index = 1; index <= x; index *= 10);
  
    while(index > 1) {
         index /= 10;
         put_char('0' + x / index);
         x = x % index;
    }
}

int sense(unsigned char muxval, int sampleX, unsigned int delay_time)
{  
    // Wait for the receive pin to settle
    ADMUX = muxval;
    
    int value_up;
    int value_down;
    int value_diff = 0;
    
    while (sampleX--) {
    
        int temp_value = 100;

        // Wait for the pin to discharge fully
        while(temp_value > 10) {
            ADCSRA |= 1 << ADSC;
            while (ADCSRA & (1 << ADSC));
            temp_value = ADCL;
            temp_value += ADCH << 8;
        }
        
        DDRA &= ~(0b11111); // Make all of the inputs inputs

        // Turn on the send pin
        set(PORTA, PA5);
        
        // Get an analog read going up
        _delay_loop_2(delay_time);
        ADCSRA |= 1 << ADSC;
        while (ADCSRA & (1 << ADSC));
        value_up = ADCL;
        value_up += ADCH << 8;
        
        // Wait to hit the "top"
        while(temp_value < 800) {
            ADCSRA |= 1 << ADSC;
            while (ADCSRA & (1 << ADSC));
            temp_value = ADCL;
            temp_value += ADCH << 8;
        }
        // Make the send pin low to measure discharge curve
        clear(PORTA, PA5);
        
        // ...and get an analog read going down.
        _delay_loop_2(delay_time);
        ADCSRA |= 1 << ADSC;
        while (ADCSRA & (1 << ADSC));
        value_down = ADCL;
        value_down += ADCH << 8;

        // Make receive pins outputs to quickly bleed off charge.
        DDRA |= 0b11111;
        value_diff += value_down - value_up;
    }
    return value_diff;
}

int main(void)
{
    //
    // main
    //

    // ADC enabled, flag cleared, prescalar set to 64 (125 kHz ADC clock)
    ADCSRA = 0b10010110;

    set(DDRA, PA6); // LED is an output

    set(PORTA, PA7); // Data out is high when idle  
    set(DDRA, PA7); // Data out is an output
    
    set(DDRA, PA5); // Capsense send is an output
    
    set(PORTB, PB2); // Enable pull-up on clock signal.

    int i;
    for(i = 0; i < 20; ++i) {
        PORTA ^= 1 << PA6;
        _delay_ms(25);
    } 
    
    while(1) {
    
        int admux;
        for (admux = 0; admux < 5; ++admux) {
            int pad = sense(admux, NUM_SAMPLES, delay_times[admux]);
            if (pad > thresholds[admux]) {
                if (buttons_down[admux] < DEBOUNCE && ++buttons_down[admux] == DEBOUNCE)
                    put_char(characters[admux]);
            } else
                buttons_down[admux] = 0;
        }
    }
}